#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <time.h>
#include <sys/time.h>
#include <math.h>
#include <complex.h>
#include <omp.h>
extern "C" {
#include <cblas.h>
}
#include <cuda_runtime.h>
#include <cublas_v2.h>

#define	ITER	1

#include "aspen_const.h"


#if TYPE_DD
#include "aspen_ddreal.h"
#endif
#if TYPE_DD_COMPLEX
#include "aspen_ddreal.h"
#include "aspen_ddcomplex.h"
#endif

#define USE_CUDA_EVENT_TIMER	0
#include "test-common.h"
#include "test.h"


template <class T> T rand_(void) { T t; return t; }

template <> float rand_<float>(void) {
  const float t = (float)rand();
  return t / RAND_MAX;
}

template <> double rand_<double>(void) {
  const double t = (double)rand();
  return t / RAND_MAX;
}

#if TYPE_DD || TYPE_DD_COMPLEX
template <> cuddreal rand_<cuddreal>(void) {
  const double t1 = rand_<double>();
  const double t2 = rand_<double>() / (((long)1)<<53);
  const cuddreal t = { t1, t2 };
  return t;
}
#endif

template <> cuFloatComplex rand_<cuFloatComplex>(void) {
  const float tx = rand_<float>(), ty = rand_<float>();
  const cuFloatComplex t = { tx, ty };
  return t;
}

template <> cuDoubleComplex rand_<cuDoubleComplex>(void) {
  const double tx = rand_<double>(), ty = rand_<double>();
  const cuDoubleComplex t = { tx, ty };
  return t;
}

#if TYPE_DD_COMPLEX
template <> cuddcomplex rand_<cuddcomplex>(void) {
  const cuddreal tx = rand_<cuddreal>(), ty = rand_<cuddreal>();
  const cuddcomplex t = { tx, ty };
  return t; }
#endif


#if TYPE_DD
void wsymv(
           const char *uplo,
           const int n,
           const cuddreal alpha, const cuddreal * A, const int lda,
           const cuddreal * x, const int incx,
           const cuddreal beta, cuddreal * y, const int incy
           )
{

  const cuddreal ZERO = { 0e0, 0e0 };
  static int num_threads; 
#pragma omp parallel
  {
#pragma omp master
    { num_threads = omp_get_num_threads(); }
  }

  size_t len;
  len = ((size_t)sizeof(cuddreal))*n;
  cuddreal *w = (cuddreal*)malloc( len );
  len = ((size_t)sizeof(cuddreal))*n*num_threads;
  cuddreal *wk = (cuddreal*)malloc( len );

  for(int j=0;j<n;j++){ y[j] *= beta; w[j] = ZERO; }
#pragma omp parallel for
  for(int i=0;i<num_threads;i++){
    cuddreal *WK = wk + i*n;
    for(int j=0;j<n;j++){ WK[j] = ZERO; }
  }

  if ( *uplo == 'u' || *uplo == 'U' ) {

#pragma omp parallel for schedule(dynamic)
    for(int i=0;i<n;i++){
      cuddreal *WK = wk + omp_get_thread_num()*n;
      cuddreal s = ZERO, u = x[i];
      for(int j=0;j<i;j++){
        cuddreal t = A[j+lda*i];
        WK[j] += t * u;
        s     += t * x[j];
      }
      s += A[i+lda*i] * x[i];
      WK[i] += s;
    }
#pragma omp parallel for schedule(dynamic)
    for(int i=0;i<n;i++){
      for(int j=0;j<num_threads;j++){
        w[i] += wk[i+j*n];
      }
    }
#pragma omp parallel for schedule(dynamic)
    for(int i=0;i<n;i++){
      y[i] = y[i] + alpha * w[i];
    }

  } else if ( *uplo == 'l' || *uplo == 'L' ) {

#pragma omp parallel for schedule(dynamic)
    for(int i=0;i<n;i++){
      cuddreal *WK = wk + omp_get_thread_num()*n;
      cuddreal s = ZERO, u = x[i];
      s += A[i+lda*i] * x[i];
      for(int j=i+1;j<n;j++){
        cuddreal t = A[j+lda*i];
        WK[j] += t * u;
        s     += t * x[j];
      }
      WK[i] += s;
    }
#pragma omp parallel for schedule(dynamic)
    for(int i=0;i<n;i++){
      for(int j=0;j<num_threads;j++){
        w[i] += wk[i+j*n];
      }
    }
#pragma omp parallel for schedule(dynamic)
    for(int i=0;i<n;i++){
      y[i] = y[i] + alpha * w[i];
    }
  }

  free(w);
  free(wk);
}

#endif
#if TYPE_DD_COMPLEX
void uhemv(
           const char *uplo,
           const int n,
           const cuddcomplex alpha, const cuddcomplex * A, const int lda,
           const cuddcomplex * x, const int incx,
           const cuddcomplex beta, cuddcomplex * y, const int incy)
{

  const cuddreal ZERO_ = { 0e0, 0e0 };
  const cuddcomplex ZERO = { ZERO_, ZERO_ };
  static int num_threads; 
#pragma omp parallel
  {
#pragma omp master
    { num_threads = omp_get_num_threads(); }
  }

  size_t len;
  len = ((size_t)sizeof(cuddcomplex))*n;
  cuddcomplex *w = (cuddcomplex*)malloc( len );
  len = ((size_t)sizeof(cuddcomplex))*n*num_threads;
  cuddcomplex *wk = (cuddcomplex*)malloc( len );
  for(int j=0;j<n;j++){ y[j] *= beta; w[j] = ZERO; }
#pragma omp parallel for
  for(int i=0;i<num_threads;i++){
    cuddcomplex *WK = wk + i*n;
    for(int j=0;j<n;j++){ WK[j] = ZERO; }
  }

  if ( *uplo == 'u' || *uplo == 'U' ) {

#pragma omp parallel for schedule(dynamic)
    for(int i=0;i<n;i++){
      cuddcomplex s = ZERO, u = x[i];
      cuddcomplex *WK = wk + omp_get_thread_num()*n;
      for(int j=0;j<i;j++){
        cuddcomplex t = A[j+lda*i];
        WK[j] += t * u;
      }
      for(int j=0;j<i;j++){
        cuddcomplex t = A[j+lda*i];
        t = Conj(t);
        s    += t * x[j];
      }
      s += A[i+lda*i] * x[i];
      WK[i] += s;
    }
#pragma omp parallel for schedule(dynamic)
    for(int i=0;i<n;i++){
      for(int j=0;j<num_threads;j++){
        w[i] += wk[i+j*n];
      }
    }
#pragma omp parallel for schedule(dynamic)
    for(int i=0;i<n;i++){
      y[i] = y[i] + alpha * w[i];
    }

  } else if ( *uplo == 'l' || *uplo == 'L' ) {

#pragma omp parallel for schedule(dynamic)
    for(int i=0;i<n;i++){
      cuddcomplex s = ZERO, u = x[i];
      cuddcomplex *WK = wk + omp_get_thread_num()*n;
      s += A[i+lda*i] * x[i];
      for(int j=i+1;j<n;j++){
        cuddcomplex t = A[j+lda*i];
        WK[j] += t * u;
      }
      for(int j=i+1;j<n;j++){
        cuddcomplex t = A[j+lda*i];
        t = Conj(t);
        s    += t * x[j];
      }
      WK[i] += s;
    }
#pragma omp parallel for schedule(dynamic)
    for(int i=0;i<n;i++){
      for(int j=0;j<num_threads;j++){
        w[i] += wk[i+j*n];
      }
    }
#pragma omp parallel for schedule(dynamic)
    for(int i=0;i<n;i++){
      y[i] = y[i] + alpha * w[i];
    }
  }

  free(w);
}
#endif


int
set_matrix_vectors( const int N, TYPE *A, const int lda, TYPE *X, TYPE *Y,
                    const enum _FUNCS func );

int
set_matrix_vectors( const int N, TYPE *A, const int lda, TYPE *X, TYPE *Y,
                    const enum _FUNCS func )
{

  srand(0);

  for(int i=0;i<N;i++) {
    for(int j=0;j<N;j++) {
      A[j+i*lda]=makeCONST<TYPE>(0);
    }
  }

  for(int i=0;i<N;i++) X[i] = rand_<TYPE>();
  for(int i=0;i<N;i++) Y[i] = rand_<TYPE>();

  for(int i=0;i<N;i++) {
    for(int j=0;j<N;j++) {
      A[j+lda*i] =  rand_<TYPE>();
    }
  }

#if TYPE_DD || TYPE_DOUBLE || TYPE_FLOAT
  if ( func == SYMV_U ) {
#pragma omp parallel for
    for(int i=0;i<N;i++) {
      for(int j=0;j<i;j++) {
        TYPE  t1  = A[j+lda*i];
        TYPE  t2  = A[i+lda*j];
        A[i+lda*j] = makeCONST<TYPE>(0);
        A[j+lda*i] = t1+t2;
#if TYPE_DD
        A[j+lda*i].x /= 2;
        A[j+lda*i].y /= 2;
#else
        A[j+lda*i] /= 2;
#endif
      }
    }
  }
  if ( func == SYMV_L ) {
#pragma omp parallel for
    for(int i=0;i<N;i++) {
      for(int j=i+1;j<N;j++) {
        TYPE  t1  = A[j+lda*i];
        TYPE  t2  = A[i+lda*j];
        A[i+lda*j] = makeCONST<TYPE>(0);
        A[j+lda*i] = t1+t2;
#if TYPE_DD
        A[j+lda*i].x /= 2;
        A[j+lda*i].y /= 2;
#else
        A[j+lda*i] /= 2;
#endif
      }
    }
  }
#else
  if ( func == HEMV_U ) {
#pragma omp parallel for
    for(int i=0;i<N;i++) {
      A[i+lda*i].y = makeCONST<TYPE_>(0);
      for(int j=0;j<i;j++) {
        TYPE  t1  = A[j+lda*i];
        TYPE  t2  = A[i+lda*j];
        A[i+lda*j] = makeCONST<TYPE>(0);
        A[j+lda*i] = t1+t2;
#if TYPE_DD_COMPLEX
        A[j+lda*i].x.x /= 2;
        A[j+lda*i].x.y /= 2;
        A[j+lda*i].y.x /= 2;
        A[j+lda*i].y.y /= 2;
#else
        A[j+lda*i].x /= 2;
        A[j+lda*i].y /= 2;
#endif
      }
    }
  }
  if ( func == HEMV_L ) {
#pragma omp parallel for
    for(int i=0;i<N;i++) {
      A[i+lda*i].y = makeCONST<TYPE_>(0);
      for(int j=i+1;j<N;j++) {
        TYPE  t1  = A[j+lda*i];
        TYPE  t2  = A[i+lda*j];
        A[i+lda*j] = makeCONST<TYPE>(0);
        A[j+lda*i] = t1+t2;
#if TYPE_DD_COMPLEX
        A[j+lda*i].x.x /= 2;
        A[j+lda*i].x.y /= 2;
        A[j+lda*i].y.x /= 2;
        A[j+lda*i].y.y /= 2;
#else
        A[j+lda*i].x /= 2;
        A[j+lda*i].y /= 2;
#endif
      }
    }
  }
#endif
}


int
main(int argc, char *argv[])
{
  TYPE	alpha, beta;

  TYPE	*A, *B, *C, *D;
  TYPE	*dA, *dB, *dC, *dD;

  int	N, M, K;
  int	LDA, LDB, LDC;
  int	i;
  int	blk;
  FILE	*fp;
  int	verify = 0;
  int	id = get_gpu_id();

  enum _FUNCS func;
  char	*func_name = (char *)"";

  cudaError_t st;

  if ( cudaSuccess != (st = cudaSetDevice(id)) ) {
    exit(1);
  }

  cudaDeviceReset();
  cudaDeviceReset();
  sleep(1);

  if ( cudaSuccess != (st = cudaSetDevice(id)) ) {
    exit(1);
  }

  cublasStatus_t stat;
  cublasHandle_t handle;
  stat = cublasCreate( &handle );


  ASPEN_init( id );

  {
    char *argv0 = argv[0];
    if ( strrchr(argv0, '/') != argv0 ) {
      argv0 = strrchr(argv0,'/')+1;
    }
#if TYPE_DD || TYPE_DOUBLE || TYPE_FLOAT
    if ( 0 == strcmp(argv0,TEST_SYMV_U) ) {
      func      = SYMV_U;
      func_name = (char *)NAME_SYMV_U;
    }
    if ( 0 == strcmp(argv0,TEST_SYMV_L) ) {
      func      = SYMV_L;
      func_name = (char *)NAME_SYMV_L;
    }
#else
    if ( 0 == strcmp(argv0,TEST_HEMV_U) ) {
      func      = HEMV_U;
      func_name = (char *)NAME_HEMV_U;
    }
    if ( 0 == strcmp(argv0,TEST_HEMV_L) ) {
      func      = HEMV_L;
      func_name = (char *)NAME_HEMV_L;
    }
#endif
  }

  if ( argc > 1 ) {
    fp=fopen(argv[1], "r");
  } else {
    fp=fopen("IN", "r");
  }
  if ( fp == NULL ) {
    fprintf(stderr,"Cannot open parameter file.\n");
    exit(1);
  }

  fscanf(fp,"%d", &N);

  if ( argc > 2 ) {
    sscanf(argv[2], "%d", &blk);
    printf("% BLK specified=%d\n", blk);
  } else {
    blk = -1; //64;
  }

  {
    verify = 1;
  }

  print_head( func_name, argc, argv );

  fflush(stdout);


  alpha =  makeCONST<TYPE>(3);
  alpha += makeCONST<TYPE>(3*EPS);
  beta  =  makeCONST<TYPE>(4);
  beta  += makeCONST<TYPE>(4*EPS);

  int N_fault=0;
  int N_max = get_device_WorkSize( id, sizeof(TYPE) );

  for(int Itr = 0; fscanf(fp,"%d", &N) > 0; Itr++ ) {

    if ( N < 1 || N > N_max ) break;


    int	M = N;

    int	LDM = ((M-1)/(256/sizeof(TYPE))+1);
    LDM *= (256/sizeof(TYPE));
    LDA = ((N-1)/(256/sizeof(TYPE))+1);
    LDA *= (256/sizeof(TYPE));

    size_t len;
    len = ((size_t)sizeof(TYPE)*M)*LDM;
    A = (TYPE *)malloc( len );
    len = ((size_t)sizeof(TYPE)*N);
    B = (TYPE *)malloc( len );
    len = ((size_t)sizeof(TYPE)*N);
    C = (TYPE *)malloc( len );
    len = ((size_t)sizeof(TYPE)*N);
    D = (TYPE *)malloc( len );

    cublasAlloc( M*LDM, sizeof(TYPE), (void**)&dA );
    cublasAlloc( N*1,   sizeof(TYPE), (void**)&dB );
    cublasAlloc( N*1,   sizeof(TYPE), (void**)&dC );
    cublasAlloc( N*1,   sizeof(TYPE), (void**)&dD );

    if(dA==NULL||dB==NULL||dC==NULL||dD==NULL) {
      fprintf(stderr, "# Fail to allocate Dev_array %dx%d, %d*4\n",
              N, LDA, N);
      fprintf(stderr, "# Fail to allocate memory %x %x %x %x\n",
              dA, dB, dC, dD);
      if(dA!=NULL) cublasFree(dA);
      if(dB!=NULL) cublasFree(dB);
      if(dC!=NULL) cublasFree(dC);
      if(dD!=NULL) cublasFree(dD);
      N_fault++;
      goto FFF;
    } else {
      N_fault=0;
    }

    printf("N= %d ", N);
    fflush(stdout);
    fflush(stdout);
    fflush(stdout);


    set_matrix_vectors( N, A, LDA, B, C, func );

    for(int i=0;i<N;i++) D[i] = C[i];


    if ( argc > 3 ) { ; } else {
      cublasSetMatrix(LDM, M, sizeof(TYPE), A, LDM, dA, LDM);
      cublasSetVector(N, sizeof(TYPE), B, 1, dB, 1);
      cublasSetVector(N, sizeof(TYPE), C, 1, dC, 1);
      cublasSetVector(N, sizeof(TYPE), D, 1, dD, 1);
    }

    fflush(stdout);
    cudaThreadSynchronize();

    for(int i=0;i<ITER;i++){

      int	incb = 1;
      int	incc = 1;
      char	*opt = (char *)"U";

      switch (func) {

#if TYPE_DD || TYPE_DOUBLE || TYPE_FLOAT
      case SYMV_U:
        ASPEN_SYMVu (blk, N, dA, LDA, dB, dC, alpha, beta);
        break;

      case SYMV_L:
        ASPEN_SYMVl (blk, N, dA, LDA, dB, dC, alpha, beta);
        break;
#endif

#if TYPE_DD_COMPLEX || TYPE_DOUBLE_COMPLEX || TYPE_FLOAT_COMPLEX
      case HEMV_U:
        ASPEN_HEMVu (blk, N, dA, LDA, dB, dC, alpha, beta);
        break;

      case HEMV_L:
        ASPEN_HEMVl (blk, N, dA, LDA, dB, dC, alpha, beta);
        break;
#endif

      }

#if 1
      cudaError_t cu_err = cudaGetLastError();
      if ( cu_err != cudaSuccess ) {
        printf("CUDA internal ERROR detected\n"); goto HHH;
      }
#endif

      if ( argc > 3 ) goto FIN;

#if 1
      if(verify){
        if(i==0&&N<50000){
          cudaThreadSynchronize();
          do_usleep(100);
          for(int j=0;j<N;j++) {
            C[j] = makeCONST<TYPE>(0);
          }
          cublasGetVector(N, sizeof(TYPE), dC, 1, C, 1);
          cudaThreadSynchronize();
          do_usleep(100);

          {

#if TYPE_DOUBLE_COMPLEX || TYPE_FLOAT_COMPLEX
#define	T(x)	(void *)x
#define	P(x)	(void *)&x
#else
#define	T(x)	x
#define	P(x)	x
#endif

#if TYPE_DD || TYPE_DD_COMPLEX
#define	OPTIONS_U	"U"
#define	OPTIONS_L	"L"
#else
#define OPTIONS_U	CblasColMajor, CblasUpper
#define OPTIONS_L	CblasColMajor, CblasLower
#endif

            switch (func) {

#if TYPE_DD || TYPE_DOUBLE || TYPE_FLOAT
            case SYMV_U:
              cblas_symv ( 
                          OPTIONS_U,
                          N, P(alpha), T(A), LDA, T(B), 1, P(beta), T(D), 1 );
              break;

            case SYMV_L:
              cblas_symv ( 
                          OPTIONS_L,
                          N, P(alpha), T(A), LDA, T(B), 1, P(beta), T(D), 1 );
              break;
#endif

#if TYPE_DD_COMPLEX || TYPE_DOUBLE_COMPLEX || TYPE_FLOAT_COMPLEX
            case HEMV_U:
              cblas_hemv ( 
                          OPTIONS_U,
                          N, P(alpha), T(A), LDA, T(B), 1, P(beta), T(D), 1 );
              break;

            case HEMV_L:
              cblas_hemv ( 
                          OPTIONS_L,
                          N, P(alpha), T(A), LDA, T(B), 1, P(beta), T(D), 1 );
              break;
#endif

            }
          }
          do_usleep(100);


#if TYPE_DOUBLE
#define	FABS	fabs
#endif
#if TYPE_FLOAT
#define	FABS	fabsf
#endif
#if TYPE_DD || TYPE_DD_COMPLEX
#define	FABS	Abs
#endif
#if TYPE_DOUBLE_COMPLEX || TYPE_FLOAT_COMPLEX 
#define	FABS	abs
#endif

#if TYPE_DOUBLE || TYPE_FLOAT
#define	CONV_TYPE	TYPE
#endif
#if TYPE_DOUBLE_COMPLEX || TYPE_FLOAT_COMPLEX
#define	CONV_TYPE	std::complex<TYPE_>
#endif
#if TYPE_DD
#define	CONV_TYPE	cuddreal
#endif
#if TYPE_DD_COMPLEX
#define	CONV_TYPE	cuddcomplex
#endif


          const double tol = 32*EPS;

#if TYPE_DD || TYPE_DD_COMPLEX
          static cuddreal t1,t2,t3;
          t1 = t2 = t3 = makeCONST<cuddreal>(0);
#else
          static TYPE_ t1,t2,t3;
          t1 = t2 = t3 = makeCONST<TYPE_>(0);
#endif
          CONV_TYPE *P = (CONV_TYPE *)C;
          CONV_TYPE *Q = (CONV_TYPE *)D;

          for(int j=0;j<N;j++){
#pragma omp parallel sections
            {
#pragma omp section
              t1 = t1 + FABS(P[j]-Q[j]);
#pragma omp section
              t2 = t2 + FABS(P[j]);
#pragma omp section
              t3 = t3 + FABS(Q[j]);
            }
          }

          double T1,T2,T3;
#if TYPE_DD || TYPE_DD_COMPLEX
          T1 = (double)(t1.x);
          T2 = (double)(t2.x);
          T3 = (double)(t3.x);
#else
          T1 = (double)(t1);
          T2 = (double)(t2);
          T3 = (double)(t3);
#endif

          printf( " | ASPEN - CBLAS | = %g\n", T1 );
          printf( " | ASPEN | = %g, | CBLAS | = %g\n", T2, T3 );
          printf( " TOL = %g ", tol * T3 );

          if ( T1 >= tol * T2 ) {
            printf( " \n" );
            printf( " ********* \n" );
            printf( " **ERROR** \n" );
            printf( " ********* \n" );
          } else {
            printf( " ++PASSED++\n" );
          }

          cudaThreadSynchronize();
          do_usleep(100);
        }
      }

#endif

    }

  EEE:;

    cudaThreadSynchronize();

    fflush(stdout);
  GGG:
    fflush(stdout);

    cublasFree(dA); //printf(":dA");
    cublasFree(dB); //printf(":dB");
    cublasFree(dC); //printf(":dC");
    cublasFree(dD); //printf(":dC");

  FFF:
    free(A); //printf(":A");
    free(B); //printf(":B");
    free(C); //printf(":C");
    free(D); //printf(":D");

    printf("\n");
    fflush(stdout);
    fflush(stdout);

    if ( N_fault > 4 ) break;
    if ( feof(fp) ) break;
  }


  cublasDestroy( handle );
  ASPEN_shutdown();
 HHH:
  cudaDeviceReset();
  fclose(fp);
 FIN:
  ;

}

