#ifndef	ASPEN_FORTRAN_H_INCLUDED

#include "aspen_version.h"
#include "aspen_types.h"


#ifdef __cplusplus
extern "C" {
#endif
  
  
  /* User must call aspen_init for initialization */
  void
  aspen_init_ ( const int * n );
  
  /* User must finalize the ASPEN library by aspen_shutdown */
  void
  aspen_shutdown_ ( void );
  
  
  void
  aspen_ssymv_ ( const char * uplo, const int * n,
                 const float * alpha, 
                 const devptr_t * a, const int * lda,
                 const devptr_t * x, const int * incx,
                 const float * beta,  
                 const devptr_t * y, const int * incy );
  void
  aspen_dsymv_ ( const char * uplo, const int * n,
                 const double * alpha, 
                 const devptr_t * a, const int * lda,
                 const devptr_t * x, const int * incx,
                 const double * beta,  
                 const devptr_t * y, const int * incy );

  void
  aspen_chemv_ ( const char * uplo, const int * n,
                 const cuFloatComplex * alpha, 
                 const devptr_t * a, const int * lda,
                 const devptr_t * x, const int * incx,
                 const cuFloatComplex * beta,  
                 const devptr_t * y, const int * incy );
  void
  aspen_zhemv_ ( const char * uplo, const int * n,
                 const cuDoubleComplex * alpha, 
                 const devptr_t * a, const int * lda,
                 const devptr_t * x, const int * incx,
                 const cuDoubleComplex * beta,  
                 const devptr_t * y, const int * incy );

  void
  aspen_wsymv_ ( const char * uplo, const int * n,
                 const cuddreal * alpha, 
                 const devptr_t * a, const int * lda,
                 const devptr_t * x, const int * incx,
                 const cuddreal * beta,  
                 const devptr_t * y, const int * incy );

  void
  aspen_uhemv_ ( const char * uplo, const int * n,
                 const cuddcomplex * alpha, 
                 const devptr_t * a, const int * lda,
                 const devptr_t * x, const int * incx,
                 const cuddcomplex * beta,  
                 const devptr_t * y, const int * incy );



#if defined(half)
  void
  aspen_haxpy_ ( const int * n,
                 const half * alpha,
                 const devptr_t * x, const int * incx,
                 const devptr_t * y, const int * incy );
#endif
  void
  aspen_saxpy_ ( const int * n,
                 const float * alpha,
                 const devptr_t * x, const int * incx,
                 const devptr_t * y, const int * incy );
  void
  aspen_daxpy_ ( const int * n,
                 double * alpha,
                 const devptr_t * x, const int * incx,
                 const devptr_t * y, const int * incy );
  void
  aspen_caxpy_ ( const int * n,
                 const cuFloatComplex * alpha,
                 const devptr_t * x, const int * incx,
                 const devptr_t * y, const int * incy );
  void
  aspen_zaxpy_ ( const int * n,
                 const cuDoubleComplex * alpha,
                 const devptr_t * x, const int * incx,
                 const devptr_t * y, const int * incy );
  void
  aspen_waxpy_ ( const int * n,
                 const cuddreal * alpha,
                 const devptr_t * x, const int * incx,
                 const devptr_t * y, const int * incy );
  void
  aspen_uaxpy_ ( int * n,
                 const cuddcomplex * alpha,
                 const devptr_t * x, const int * incx,
                 const devptr_t * y, const int * incy );
 

#if defined(half)
  void
  aspen_haxpby_ ( const int * n,
                  const half * alpha,
                  const devptr_t * x, const int * incx,
                  const half * beta,
                  const devptr_t * y, const int * incy );
#endif
  void
  aspen_saxpby_ ( const int * n,
                  const float * alpha,
                  const devptr_t * x, const int * incx,
                  const float * beta,
                  const devptr_t * y, const int * incy );
  void
  aspen_daxpby_ ( const int * n,
                  const double * alpha,
                  const devptr_t * x, const int * incx,
                  const double * beta,
                  const devptr_t * y, const int * incy );
  void
  aspen_caxpby_ ( const int * n,
                  const cuFloatComplex * alpha,
                  const devptr_t * x, const int * incx,
                  const cuFloatComplex * beta,
                  const devptr_t * y, const int * incy );
  void
  aspen_zaxpby_ ( const int * n,
                  const cuDoubleComplex * alpha,
                  const devptr_t * x, const int * incx,
                  const cuDoubleComplex * beta,
                  const devptr_t * y, const int * incy );
  void
  aspen_waxpby_ ( const int * n,
                  const cuddreal * alpha,
                  const devptr_t * x, const int * incx,
                  const cuddreal * beta,
                  const devptr_t * y, const int * incy );
  void
  aspen_uaxpby_ ( const int * n,
                  const cuddcomplex * alpha,
                  const devptr_t * x, const int * incx,
                  const cuddcomplex * beta,
                  const devptr_t * y, const int * incy );


#if defined(half)
  void
  aspen_hswap_ ( const int * n,
                 const devptr_t * x, const int * incx,
                 const devptr_t * y, const int * incy );
#endif
  void
  aspen_sswap_ ( const int * n,
                 const devptr_t * x, const int * incx,
                 const devptr_t * y, const int * incy );
  void
  aspen_dswap_ ( const int n,
                 const devptr_t * x, const int incx,
                 const devptr_t * y, const int incy );
  void
  aspen_cswap_ ( const int * n,
                 const devptr_t * x, const int * incx,
                 const devptr_t * y, const int * incy );
  void
  aspen_zswap_ ( const int * n,
                 const devptr_t * x, const int * incx,
                 const devptr_t * y, const int * incy );
  void
  aspen_wswap_ ( const int * n,
                 const devptr_t * x, const int * incx,
                 const devptr_t * y, const int * incy );
  void
  aspen_uswap_ ( const int n,
                 const devptr_t * x, const int * incx,
                 const devptr_t * y, const int * incy );


#if defined(half)
  void
  aspen_hcopy_ ( const int * n,
                 const devptr_t * x, const int * incx,
                 const devptr_t * y, const int * incy );
#endif
  void
  aspen_scopy_ ( const int * n,
                 const devptr_t * x, const int * incx,
                 const devptr_t * y, const int * incy );
  void
  aspen_dcopy_ ( const int * n,
                 const devptr_t * x, const int * incx,
                 const devptr_t * y, const int * incy );
  void
  aspen_ccopy_ ( const int * n,
                 const devptr_t * x, const int * incx,
                 const devptr_t * y, const int * incy );
  void
  aspen_zcopy_ ( const int * n,
                 const devptr_t * x, const int * incx,
                 const devptr_t * y, const int * incy );
  void
  aspen_wcopy_ ( const int * n,
                 const devptr_t * x, const int * incx,
                 const devptr_t * y, const int * incy );
  void
  aspen_ucopy_ ( const int * n,
                 const devptr_t * x, const int * incx,
                 const devptr_t * y, const int * incy );

#if defined(half)
  void
  aspen_hscal_ ( const int * n,
                 const half * alpha,
                 const devptr_t * x, const int * incx );
#endif
  void
  aspen_sscal_ ( const int * n,
                 const float * alpha,
                 const devptr_t * x, const int * incx );
  void
  aspen_dscal_ ( const int * n,
                 const double * alpha,
                 const devptr_t * x, const int * incx );
  void
  aspen_cscal_ ( const int * n,
                 const cuFloatComplex * alpha,
                 const devptr_t * x, const int * incx );
  void
  aspen_zscal_ ( const int * n,
                 const cuDoubleComplex * alpha,
                 const devptr_t * x, const int * incx );
  void
  aspen_wscal_ ( const int * n,
                 const cuddreal * alpha,
                 const devptr_t * x, const int * incx );
  void
  aspen_uscal_ ( const int * n,
                 const cuddcomplex * alpha,
                 const devptr_t * x, const int * incx );
  

#if defined(half)
  void
  aspen_hzero_ ( const devptr_t * x, const int * n );
#endif
  void
  aspen_szero_ ( const devptr_t * x, const int * n );
  void
  aspen_dzero_ ( const devptr_t * x, const int * n );
  void
  aspen_czero_ ( const devptr_t * x, const int * n );
  void
  aspen_zzero_ ( const devptr_t * x, const int * n );
  void
  aspen_wzero_ ( const devptr_t * x, const int * n );
  void
  aspen_uzero_ ( const devptr_t * x, const int * n );
 

#ifdef __cplusplus
}
#endif

#define	ASPEN_FORTRAN_H_INCLUDED	1
#endif

