#ifndef ASPEN_HALF_H_INCLUDED

#if defined(ASPEN_HALF_ENABLED)
#  undef ASPEN_HALF_ENABLED
#endif

#if defined(CURRENT_GPU)
#  if CURRENT_GPU==530 || CURRENT_GPU==600 || CURRENT_GPU==620
#    if CUDA_VERSION>7000
#      define ASPEN_HALF_ENABLED	1
#    else
#      define ASPEN_HALF_ENABLED	0
#    endif
#  else
#    define ASPEN_HALF_ENABLED	0
#  endif
#else
#  define ASPEN_HALF_ENABLED	0
#endif

#if ASPEN_HALF_ENABLED

#include <cuda_fp16.h>


#ifdef __cplusplus

__host__ __device__ __forceinline__ bool
operator== ( const half a, const half b )
{
  unsigned short c, d;
  c = *((unsigned short *)&a);
  d = *((unsigned short *)&b);
  return ( c == d );
}

__host__ __device__ __forceinline__ bool
operator!= ( const half a, const half b )
{
  unsigned short c, d;
  c = *((unsigned short *)&a);
  d = *((unsigned short *)&b);
  return ( c != d );
}

__device__ __forceinline__ half
operator+ ( const half a )
{
  return a;
}

__device__ __forceinline__ half
operator+ ( const half a,  const half b )
{
  return __hadd (a, b);
}

__device__ __forceinline__ void
operator+= ( half &a,  const half b )
{
  a = __hadd (a, b);
}

__device__ __forceinline__ half
operator- ( const half a )
{
  return __hneg (a);
}

__device__ __forceinline__ half
operator- ( const half a,  const half b )
{
  return __hsub (a, b);
}

__device__ __forceinline__ void
operator-= ( half &a,  const half  b)
{
  a = __hsub (a, b);
}

__device__ __forceinline__ half
operator* ( const half a,  const half b )
{
  return __hmul (a, b);
}

__device__ __forceinline__ void
operator*= ( half &a,  const half b )
{
  a = __hmul (a, b);
}

__device__ __forceinline__ half
operator/ ( const half a,  const half b )
{
  return hdiv (a, b);
}

__device__ __forceinline__ void
operator/= ( half &a,  const half b )
{
  a = hdiv (a, b);
}

#endif

#endif

#define ASPEN_HALF_H_INCLUDED	1
#endif

