#include <stdio.h>
#include <stdlib.h>
#include <string.h>

int
_min_ (int x, int y) {
  return (x<y)?x:y;
}

int
_max_ (int x, int y) {
  return (x>y)?x:y;
}

int
_ceil_ (int x, int y) {
  return ((x-1)/y+1)*y;
}

int
_floor_ (int x, int y) {
  return (x/y)*y;
}


/*
  # ============================================================

  #	BX  Num of the Threads / ThreadBlock
  #	CX  Num of the Registers / Thread
  #	MX  [B] Minimum spend of Shared Memory / ThreadBlock
  #	CG  Compute Capability Generation (100, 200, 300, 500)
  #
  # ============================================================
*/

int
main( int argc, char * argv[] ) {

  if ( argc < 4 ) {
    printf("0(0)\n");
    exit(1);
  }

  int BX = atoi(argv[1]);
  int CX = atoi(argv[2]);
  int MX = atoi(argv[3]);
  int CG = atoi(argv[4]);

  // # ============================================================

  int SHMEM_CAPACITY_KB = 48;
  switch ( CG ) {

  case 100:
    SHMEM_CAPACITY_KB = 16;
    break;

  case 200:
  case 300:
  case 350:
    SHMEM_CAPACITY_KB = 48;
    break;

  case 530:
  case 600:
  case 620:
    SHMEM_CAPACITY_KB = 64;
    break;

  case 500:
  case 520:
  case 610:
    SHMEM_CAPACITY_KB = 96;
    break;

  default:
    SHMEM_CAPACITY_KB = 48;

  }

  int SX = SHMEM_CAPACITY_KB * 1024;

  if ( CG == 200 || CG == 300 || CG == 350 ) {
    if ( strcmp( argv[0], "get_mult" ) ) {
      SX = 16 * 1024;
    }
    if ( strcmp( argv[0], "get_mult32" ) ) {
      SX = 32 * 1024;
    }
    if ( strcmp( argv[0], "get_mult48" ) ) {
      SX = 48 * 1024;
    }
  }

  // # ============================================================

  int WARP            = 32;

  int MAX_REGS        = -1;
  int MAX_BLK         = -1;
  int MAX_WARPS       = -1;
  int SHMEM_UNIT_SIZE = -1;
  int REG_UNIT_SIZE   = -1;
  int GRANURARITY     = -1;

  // # ============================================================

  switch ( CG ) {
  case 130:
    // # for Tesla
    MAX_REGS        = 16384;
    MAX_BLK         = 8;
    MAX_WARPS       = 32;
    SHMEM_UNIT_SIZE = 512;
    REG_UNIT_SIZE   = 512;
    GRANURARITY     = 2;
    break;

  case 200:
    // # for Fermi
    MAX_REGS        = 32768;
    MAX_BLK         = 8;
    MAX_WARPS       = 48;
    SHMEM_UNIT_SIZE = 128;
    switch ( CX ) {
    case 21:
    case 22:
    case 29:
    case 30:
    case 37:
    case 38:
    case 45:
    case 46:
      REG_UNIT_SIZE = 128;
      break;
    default:
      REG_UNIT_SIZE=64;
      break;
    }
    GRANURARITY     = 2;
    break;

  case 300:
    // # for Kepler
  case 350:
    // # for Kepler2
    MAX_REGS        = 65536;
    MAX_BLK         = 16;
    MAX_WARPS       = 64;
    SHMEM_UNIT_SIZE = 256;
    REG_UNIT_SIZE   = 256;
    GRANURARITY     = 4;
    break;

  case 500:
    // # for Maxwell
  case 520:
    // # for Maxwell2
  case 530:
    // # for Maxwell3
    MAX_REGS        = 65536;
    MAX_BLK         = 32;
    MAX_WARPS       = 64;
    SHMEM_UNIT_SIZE = 256;
    REG_UNIT_SIZE   = 256;
    GRANURARITY     = 4;
    break;

  case 600:
    // # for Pascal
  case 610:
    // # for Pascal1
  case 620:
    // # for Pascal2
    MAX_REGS        = 65536;
    MAX_BLK         = 32;
    MAX_WARPS       = 64;
    SHMEM_UNIT_SIZE = 256;
    REG_UNIT_SIZE   = 256;
    GRANURARITY     = 4;
    break;

  default:
    printf("0(0)\n");
    exit(1);
    break;
  }

  // # ============================================================

  if ( MAX_REGS < 0 ) {
    printf("0(0)\n");
    exit(1);
  }

  if ( SX < MX ) {
    printf("0(0)\n");
    exit(1);
  }

  // # ============================================================

  int X0, X1;
  int Y0, Y1, Y2, Y3, Y4, Y5, Y6;

  // # ============================================================

  X0 = SX / SHMEM_UNIT_SIZE;
  X1 = ((MX-1) / SHMEM_UNIT_SIZE) + 1;

  // # ============================================================

  Y0 = CX * WARP;
  Y1 = _ceil_ ( Y0, REG_UNIT_SIZE );
  Y3= ((BX-1)/WARP) + 1;

  if ( CG < 200 ) {

    Y2 = _ceil_ ( Y3, 2 );
    Y4 = _ceil_ ( Y2*Y0, REG_UNIT_SIZE );
    Y5 = MAX_REGS;

  } else {

    Y2 = MAX_REGS/Y1;
    Y4 = Y3;
    Y5 = _floor_ ( Y2, GRANURARITY );

  }

  // # ============================================================

  int M0 = _min_ ( _min_ ( Y5/Y4, X0/X1 ), _min_ ( MAX_WARPS/Y3, MAX_BLK ) );

  // # ============================================================

  char *M_PATTERN[2];
  M_PATTERN[0] = (char *)malloc(1024); *M_PATTERN[0] = 0;
  M_PATTERN[1] = (char *)malloc(1024); *M_PATTERN[1] = 0;
  int ptr=0;

  for( int MM = M0; MM > 0; MM-- ) {

    if ( MM == M0 || (MM+1)*(X0/MM) > X0 ) {

      int OO = (MM * Y3 * 100) / MAX_WARPS;
      sprintf( M_PATTERN[ptr],
               "%d(%d) %s", MM, OO, M_PATTERN[1-ptr] );
      ptr=1-ptr;

    }
  }

  // # ============================================================

  ptr=1-ptr;
  printf("%s\n", M_PATTERN[ptr]);

  // # ============================================================

  free(M_PATTERN[0]);
  free(M_PATTERN[1]);

  exit (0);

}

