#!/bin/sh

echo_Message()
{
	echo '<--/****************************************'
	echo 'Automatic Performance Tuning for SSYMV-U'
	date
	echo -n 'Host on '; echo `hostname`
	echo 'Device is '$DEVICE
	if [ x${1} = xbegin ]; then
		echo 'Start.'
	fi
	if [ x${1} = xend ]; then
		echo 'Successfully completed.'
	fi
	if [ x${1} = xterminate ]; then
		echo 'Terminated unfortunately.'
	fi
	echo '****************************************/-->'
}


export LANG=C
LOG=log-ssymvu-AT1X.`date +%Y.%m.%d.%T`
WORK_DIR=ssymvu-current

if [ x${CUDA_PATH} != x ]; then
        export LD_LIBRARY_PATH=$CUDA_PATH/lib64:$LD_LIBRARY_PATH
fi
export LD_LIBRARY_PATH=`pwd`/../lib:$LD_LIBRARY_PATH

cd ../src; make get_dev_info; cd ../tuning
../src/get_dev_info $ASPEN_GPU_ID > DEV_INFO

DEVICE=`awk '/DEVICE=/{ print $2 }' DEV_INFO`
MP=`awk '/MP=/{ print $2 }' DEV_INFO`
CG=`awk '/CG=/{ print $2 }' DEV_INFO`
MAXDIM=`awk '/MAXDIM2=/{ print $2 }' DEV_INFO`


cd ../src; make get_mult get_mult32 get_mult48; cd ../tuning

echo_Message begin

/bin/sh ./touch_header.sh

mkdir $LOG
if [ -f $WORK_DIR ]; then
	\rm -rf $WORK_DIR
fi
if [ -h $WORK_DIR ]; then
	\rm $WORK_DIR
fi
if [ -d $WORK_DIR ]; then
	\rm -rf $WORK_DIR
fi
ln -s $LOG $WORK_DIR

echo 'Tuning processes are recorded on '$LOG'.'

cd $WORK_DIR
cd ../../src; make get_dev_info; cd ../tuning/$WORK_DIR
ln -s ../../src/get_dev_info .

tar -C ../.. -cf - src template | tar -xf -

cat symv_upper_atomic_template.incl

/bin/sh ../d.sh > OK-pat-symv

touch done_pattern
\rm done_pattern

/bin/sh ../ssymvu_a.sh | tee $LOG
/bin/sh ../ssymvu_b.sh $LOG
/bin/sh ../ssymvu_b2.sh $LOG-X
/bin/sh ../ssymvu_c.sh
/bin/sh ../d_filter.sh ssymv-upper-auto3.h ssymv-upper-auto2.h

echo '#if 0'             > ssymv-upper-auto_.h
echo_Message            >> ssymv-upper-auto_.h
cat ../DEV_INFO         >> ssymv-upper-auto_.h
echo '<--'              >> ssymv-upper-auto_.h
cat ../CURRENT_GPU      >> ssymv-upper-auto_.h
echo '-->'              >> ssymv-upper-auto_.h
echo '#endif'           >> ssymv-upper-auto_.h
cat ssymv-upper-auto.h  >> ssymv-upper-auto_.h
mv ssymv-upper-auto_.h ssymv-upper-auto.h
cp ssymv-upper-auto.h ..

echo '#if 0'             > ssymv-upper-auto_.h
echo_Message            >> ssymv-upper-auto_.h
cat ../DEV_INFO         >> ssymv-upper-auto_.h
echo '<--'              >> ssymv-upper-auto_.h
cat ../CURRENT_GPU      >> ssymv-upper-auto_.h
echo '-->'              >> ssymv-upper-auto_.h
echo '#endif'           >> ssymv-upper-auto_.h
cat ssymv-upper-auto2.h >> ssymv-upper-auto_.h
mv ssymv-upper-auto_.h ssymv-upper-auto2.h
cp ssymv-upper-auto2.h ..

cat ssymv-upper-auto.h
cat ssymv-upper-auto2.h

cd ../../src

\rm ssymv_upper.cu_o
make

cd ../bench

\rm test-s.o test2-s.o
make 

timeout -s KILL 3600 ./test-ssymv-u IN-medium
timeout -s KILL 7200 ./test2-ssymv-u IN-medium

cd ../tuning

touch .done-ssymvu

echo_Message end

exit 0

