#!/bin/sh

echo_Message()
{
	echo '<--/****************************************'
	echo 'Automatic Performance Tuning for UHEMV-U'
	date
	echo -n 'Host on '; echo `hostname`
	echo 'Device is '$DEVICE
	if [ x${1} = xbegin ]; then
		echo 'Start.'
	fi
	if [ x${1} = xend ]; then
		echo 'Successfully completed.'
	fi
	if [ x${1} = xterminate ]; then
		echo 'Terminated unfortunately.'
	fi
	echo '****************************************/-->'
}


export LANG=C
LOG=log-uhemvu-AT1X.`date +%Y.%m.%d.%T`
WORK_DIR=uhemvu-current

if [ x${CUDA_PATH} != x ]; then
        export LD_LIBRARY_PATH=$CUDA_PATH/lib64:$LD_LIBRARY_PATH
fi
export LD_LIBRARY_PATH=`pwd`/../lib:$LD_LIBRARY_PATH

cd ../src; make get_dev_info; cd ../tuning
../src/get_dev_info $ASPEN_GPU_ID > DEV_INFO

DEVICE=`awk '/DEVICE=/{ print $2 }' DEV_INFO`
MP=`awk '/MP=/{ print $2 }' DEV_INFO`
CG=`awk '/CG=/{ print $2 }' DEV_INFO`
MAXDIM=`awk '/MAXDIM4=/{ print $2 }' DEV_INFO`


cd ../src; make get_mult get_mult32 get_mult48; cd ../tuning

echo_Message begin

/bin/sh ./touch_header.sh

mkdir $LOG
if [ -f $WORK_DIR ]; then
	\rm -rf $WORK_DIR
fi
if [ -h $WORK_DIR ]; then
	\rm $WORK_DIR
fi
if [ -d $WORK_DIR ]; then
	\rm -rf $WORK_DIR
fi
ln -s $LOG $WORK_DIR

echo 'Tuning processes are recorded on '$LOG'.'

cd $WORK_DIR
cd ../../src; make get_dev_info; cd ../tuning/$WORK_DIR
ln -s ../../src/get_dev_info .

tar -C ../.. -cf - src template | tar -xf -

cat symv_upper_atomic_template.incl

/bin/sh ../d.sh > OK-pat-symv

touch done_pattern
\rm done_pattern

/bin/sh ../uhemvu_a.sh | tee $LOG
/bin/sh ../uhemvu_b.sh $LOG
/bin/sh ../uhemvu_b2.sh $LOG-X
/bin/sh ../uhemvu_c.sh
/bin/sh ../d_filter.sh uhemv-upper-auto3.h uhemv-upper-auto2.h

echo '#if 0'             > uhemv-upper-auto_.h
echo_Message            >> uhemv-upper-auto_.h
cat ../DEV_INFO         >> uhemv-upper-auto_.h
echo '<--'              >> uhemv-upper-auto_.h
cat ../CURRENT_GPU      >> uhemv-upper-auto_.h
echo '-->'              >> uhemv-upper-auto_.h
echo '#endif'           >> uhemv-upper-auto_.h
cat uhemv-upper-auto.h  >> uhemv-upper-auto_.h
mv uhemv-upper-auto_.h uhemv-upper-auto.h
cp uhemv-upper-auto.h ..

echo '#if 0'             > uhemv-upper-auto_.h
echo_Message            >> uhemv-upper-auto_.h
cat ../DEV_INFO         >> uhemv-upper-auto_.h
echo '<--'              >> uhemv-upper-auto_.h
cat ../CURRENT_GPU      >> uhemv-upper-auto_.h
echo '-->'              >> uhemv-upper-auto_.h
echo '#endif'           >> uhemv-upper-auto_.h
cat uhemv-upper-auto2.h >> uhemv-upper-auto_.h
mv uhemv-upper-auto_.h uhemv-upper-auto2.h
cp uhemv-upper-auto2.h ..

cat uhemv-upper-auto.h
cat uhemv-upper-auto2.h

cd ../../src

\rm uhemv_upper.cu_o
make

cd ../bench

\rm test-u.o test2-u.o
make 

timeout -s KILL 3600 ./test-uhemv-u IN-medium
timeout -s KILL 7200 ./test2-uhemv-u IN-medium

cd ../tuning

touch .done-uhemvu

echo_Message end

exit 0

