#!/bin/sh

echo_Message()
{
	echo '<--/****************************************'
	echo 'Automatic Performance Tuning for ZHEMV-L'
	date
	echo -n 'Host on '; echo `hostname`
	echo 'Device is '$DEVICE
	if [ x${1} = xbegin ]; then
		echo 'Start.'
	fi
	if [ x${1} = xend ]; then
		echo 'Successfully completed.'
	fi
	if [ x${1} = xterminate ]; then
		echo 'Terminated unfortunately.'
	fi
	echo '****************************************/-->'
}


export LANG=C
LOG=log-zhemvl-AT1X.`date +%Y.%m.%d.%T`
WORK_DIR=zhemvl-current

if [ x${CUDA_PATH} != x ]; then
        export LD_LIBRARY_PATH=$CUDA_PATH/lib64:$LD_LIBRARY_PATH
fi
export LD_LIBRARY_PATH=`pwd`/../lib:$LD_LIBRARY_PATH

cd ../src; make get_dev_info; cd ../tuning
../src/get_dev_info $ASPEN_GPU_ID > DEV_INFO

DEVICE=`awk '/DEVICE=/{ print $2 }' DEV_INFO`
MP=`awk '/MP=/{ print $2 }' DEV_INFO`
CG=`awk '/CG=/{ print $2 }' DEV_INFO`
MAXDIM=`awk '/MAXDIM3=/{ print $2 }' DEV_INFO`


cd ../src; make get_mult get_mult32 get_mult48; cd ../tuning

echo_Message begin

/bin/sh ./touch_header.sh

mkdir $LOG
if [ -f $WORK_DIR ]; then
	\rm -rf $WORK_DIR
fi
if [ -h $WORK_DIR ]; then
	\rm $WORK_DIR
fi
if [ -d $WORK_DIR ]; then
	\rm -rf $WORK_DIR
fi
ln -s $LOG $WORK_DIR

echo 'Tuning processes are recorded on '$LOG'.'

cd $WORK_DIR
cd ../../src; make get_dev_info; cd ../tuning/$WORK_DIR
ln -s ../../src/get_dev_info .

tar -C ../.. -cf - src template | tar -xf -

cat symv_lower_atomic_template.incl

/bin/sh ../d.sh > OK-pat-symv

touch done_pattern
\rm done_pattern

/bin/sh ../zhemvl_a.sh | tee $LOG
/bin/sh ../zhemvl_b.sh $LOG
/bin/sh ../zhemvl_b2.sh $LOG-X
/bin/sh ../zhemvl_c.sh
/bin/sh ../d_filter.sh zhemv-lower-auto3.h zhemv-lower-auto2.h

echo '#if 0'             > zhemv-lower-auto_.h
echo_Message            >> zhemv-lower-auto_.h
cat ../DEV_INFO         >> zhemv-lower-auto_.h
echo '<--'              >> zhemv-lower-auto_.h
cat ../CURRENT_GPU      >> zhemv-lower-auto_.h
echo '-->'              >> zhemv-lower-auto_.h
echo '#endif'           >> zhemv-lower-auto_.h
cat zhemv-lower-auto.h  >> zhemv-lower-auto_.h
mv zhemv-lower-auto_.h zhemv-lower-auto.h
cp zhemv-lower-auto.h ..

echo '#if 0'             > zhemv-lower-auto_.h
echo_Message            >> zhemv-lower-auto_.h
cat ../DEV_INFO         >> zhemv-lower-auto_.h
echo '<--'              >> zhemv-lower-auto_.h
cat ../CURRENT_GPU      >> zhemv-lower-auto_.h
echo '-->'              >> zhemv-lower-auto_.h
echo '#endif'           >> zhemv-lower-auto_.h
cat zhemv-lower-auto2.h >> zhemv-lower-auto_.h
mv zhemv-lower-auto_.h zhemv-lower-auto2.h
cp zhemv-lower-auto2.h ..

cat zhemv-lower-auto.h
cat zhemv-lower-auto2.h

cd ../../src

\rm zhemv_lower.cu_o
make

cd ../bench

\rm test-z.o test2-z.o
make 

timeout -s KILL 3600 ./test-zhemv-l IN-medium
timeout -s KILL 7200 ./test2-zhemv-l IN-medium

cd ../tuning

touch .done-zhemvl

echo_Message end

exit 0

