#!/bin/sh

echo_Message()
{
	echo '<--/****************************************'
	echo 'Automatic Performance Tuning for ZHEMV-U'
	date
	echo -n 'Host on '; echo `hostname`
	echo 'Device is '$DEVICE
	if [ x${1} = xbegin ]; then
		echo 'Start.'
	fi
	if [ x${1} = xend ]; then
		echo 'Successfully completed.'
	fi
	if [ x${1} = xterminate ]; then
		echo 'Terminated unfortunately.'
	fi
	echo '****************************************/-->'
}


export LANG=C
LOG=log-zhemvu-AT1X.`date +%Y.%m.%d.%T`
WORK_DIR=zhemvu-current

if [ x${CUDA_PATH} != x ]; then
        export LD_LIBRARY_PATH=$CUDA_PATH/lib64:$LD_LIBRARY_PATH
fi
export LD_LIBRARY_PATH=`pwd`/../lib:$LD_LIBRARY_PATH

cd ../src; make get_dev_info; cd ../tuning
../src/get_dev_info $ASPEN_GPU_ID > DEV_INFO

DEVICE=`awk '/DEVICE=/{ print $2 }' DEV_INFO`
MP=`awk '/MP=/{ print $2 }' DEV_INFO`
CG=`awk '/CG=/{ print $2 }' DEV_INFO`
MAXDIM=`awk '/MAXDIM3=/{ print $2 }' DEV_INFO`


cd ../src; make get_mult get_mult32 get_mult48; cd ../tuning

echo_Message begin

/bin/sh ./touch_header.sh

mkdir $LOG
if [ -f $WORK_DIR ]; then
	\rm -rf $WORK_DIR
fi
if [ -h $WORK_DIR ]; then
	\rm $WORK_DIR
fi
if [ -d $WORK_DIR ]; then
	\rm -rf $WORK_DIR
fi
ln -s $LOG $WORK_DIR

echo 'Tuning processes are recorded on '$LOG'.'

cd $WORK_DIR
cd ../../src; make get_dev_info; cd ../tuning/$WORK_DIR
ln -s ../../src/get_dev_info .

tar -C ../.. -cf - src template | tar -xf -

cat symv_upper_atomic_template.incl

/bin/sh ../d.sh > OK-pat-symv

touch done_pattern
\rm done_pattern

/bin/sh ../zhemvu_a.sh | tee $LOG
/bin/sh ../zhemvu_b.sh $LOG
/bin/sh ../zhemvu_b2.sh $LOG-X
/bin/sh ../zhemvu_c.sh
/bin/sh ../d_filter.sh zhemv-upper-auto3.h zhemv-upper-auto2.h

echo '#if 0'             > zhemv-upper-auto_.h
echo_Message            >> zhemv-upper-auto_.h
cat ../DEV_INFO         >> zhemv-upper-auto_.h
echo '<--'              >> zhemv-upper-auto_.h
cat ../CURRENT_GPU      >> zhemv-upper-auto_.h
echo '-->'              >> zhemv-upper-auto_.h
echo '#endif'           >> zhemv-upper-auto_.h
cat zhemv-upper-auto.h  >> zhemv-upper-auto_.h
mv zhemv-upper-auto_.h zhemv-upper-auto.h
cp zhemv-upper-auto.h ..

echo '#if 0'             > zhemv-upper-auto_.h
echo_Message            >> zhemv-upper-auto_.h
cat ../DEV_INFO         >> zhemv-upper-auto_.h
echo '<--'              >> zhemv-upper-auto_.h
cat ../CURRENT_GPU      >> zhemv-upper-auto_.h
echo '-->'              >> zhemv-upper-auto_.h
echo '#endif'           >> zhemv-upper-auto_.h
cat zhemv-upper-auto2.h >> zhemv-upper-auto_.h
mv zhemv-upper-auto_.h zhemv-upper-auto2.h
cp zhemv-upper-auto2.h ..

cat zhemv-upper-auto.h
cat zhemv-upper-auto2.h

cd ../../src

\rm zhemv_upper.cu_o
make

cd ../bench

\rm test-z.o test2-z.o
make 

timeout -s KILL 4   ./test-zhemv-u IN-medium >& /dev/null
timeout -s KILL 600 ./test-zhemv-u IN-medium
if [ $? -eq 137 ]; then
timeout -s KILL 600 ./test-zhemv-u IN-medium
fi

timeout -s KILL 4    ./test2-zhemv-u IN-medium >& /dev/null
timeout -s KILL 3600 ./test2-zhemv-u IN-medium
if [ $? -eq 137 ]; then
timeout -s KILL 3600 ./test2-zhemv-u IN-medium
fi

cd ../tuning

touch .done-zhemvu

echo_Message end

exit 0

